/* 
vim:expandtab:softtabstop=2:tabstop=2:shiftwidth=2:nowrap:ruler 
*/
/*
Copyright (c) 2015-2016, iwrite authors 
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include "item_fsum.h"
#include "database_field.h"
#include "mainw.h"

struct sum_dialog
{
  GtkEntry*                             m_color_entry;
  GtkButton*                            m_color_button;
  GtkEntry*                             m_font_entry;
  GtkButton*                            m_font_button; 
  GtkCheckButton*                       m_border_checkbutton;
  GtkComboBoxText*                      m_border_style_comboboxtext;
  GtkSpinButton*                        m_border_thickness_spinbutton;
  GtkEntry*                             m_border_color_entry;
  GtkButton*                            m_border_color_button;
  GtkCheckButton*                       m_fill_color_checkbutton;
  GtkEntry*                             m_fill_color_entry;
  GtkButton*                            m_fill_color_button; 
  GtkComboBoxText*                      m_type_comboboxtext;
  GtkEntry*                             m_pattern_entry;
  GtkEntry*                             m_locale_entry;
  GtkEntry*                             m_group_entry;
  GtkEntry*                             m_field_entry;
  GtkButton*                            m_field_button; 
};

static void
item_fsum_field_button_cb(
  GtkWidget *                           i_widget,
  gpointer                              i_data)
{
  struct sum_dialog*                    l_fsum;
  char*                                 l_attribute;

  l_attribute= 0;

  l_fsum= (struct sum_dialog*)i_data;

  database_field_property(
    &l_attribute, 
    &(*g_mainw).m_report.m_aspect,
    &(*g_mainw).m_report.m_query);

  if (l_attribute)
  {
    gtk_entry_set_text((*l_fsum).m_field_entry, l_attribute);
  }

  if (l_attribute)
  {
    g_free(l_attribute);
  }

  return;
}

static void
item_fsum_color_button_cb(
  GtkWidget *                           i_widget,
  gpointer                              i_data)
{
  GdkRGBA                               l_color;
  GtkWidget*                            l_dialog;
  int                                   l_exit;
  struct sum_dialog*                    l_sum;
  gchar*                                l_rgba;
  gchar const*                          l_text;

  l_dialog= 0;
  memset(&l_color, 0, sizeof(l_color));

  do
  {

    l_sum= (struct sum_dialog*)i_data;

    l_dialog= gtk_color_chooser_dialog_new("Border Color", get_main_window());

    l_text= gtk_entry_get_text((*l_sum).m_color_entry);
    gdk_rgba_parse(&l_color, l_text);
    gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(l_dialog), &l_color);

    gtk_window_set_modal((GtkWindow*)l_dialog, 1);
    l_exit= gtk_dialog_run(GTK_DIALOG(l_dialog));

    if (GTK_RESPONSE_OK != l_exit)
    {
      break;
    }

    gtk_color_chooser_get_rgba(GTK_COLOR_CHOOSER(l_dialog), &l_color);

    l_rgba= gdk_rgba_to_string(&l_color);
    gtk_entry_set_text((*l_sum).m_color_entry, l_rgba);
    g_free(l_rgba);

  }while(0);

  if (l_dialog)
  {
    gtk_widget_destroy((GtkWidget*)l_dialog);
  }
  
  return;
}

static void
item_fsum_border_color_button_cb(
  GtkWidget *                           i_widget,
  gpointer                              i_data)
{
  GdkRGBA                               l_color;
  GtkWidget*                            l_dialog;
  int                                   l_exit;
  struct sum_dialog*                    l_sum;
  gchar*                                l_rgba;
  gchar const*                          l_text;

  l_dialog= 0;
  memset(&l_color, 0, sizeof(l_color));

  do
  {

    l_sum= (struct sum_dialog*)i_data;

    l_dialog= gtk_color_chooser_dialog_new("Border Color", get_main_window());

    l_text= gtk_entry_get_text((*l_sum).m_border_color_entry);
    gdk_rgba_parse(&l_color, l_text);
    gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(l_dialog), &l_color);

    gtk_window_set_modal((GtkWindow*)l_dialog, 1);
    l_exit= gtk_dialog_run(GTK_DIALOG(l_dialog));

    if (GTK_RESPONSE_OK != l_exit)
    {
      break;
    }

    gtk_color_chooser_get_rgba(GTK_COLOR_CHOOSER(l_dialog), &l_color);

    l_rgba= gdk_rgba_to_string(&l_color);
    gtk_entry_set_text((*l_sum).m_border_color_entry, l_rgba);
    g_free(l_rgba);

  }while(0);

  if (l_dialog)
  {
    gtk_widget_destroy((GtkWidget*)l_dialog);
  }
  
  return;
}

static void
item_fsum_fill_color_button_cb(
  GtkWidget *                           i_widget,
  gpointer                              i_data)
{
  GdkRGBA                               l_color;
  GtkWidget*                            l_dialog;
  int                                   l_exit;
  struct sum_dialog*                    l_sum;
  gchar*                                l_rgba;
  gchar const*                          l_text;

  l_dialog= 0;
  memset(&l_color, 0, sizeof(l_color));

  do
  {

    l_sum= (struct sum_dialog*)i_data;

    l_dialog= gtk_color_chooser_dialog_new("Fill Color", get_main_window());
    gtk_color_chooser_set_use_alpha(GTK_COLOR_CHOOSER(l_dialog), 1);

    l_text= gtk_entry_get_text((*l_sum).m_fill_color_entry);
    gdk_rgba_parse(&l_color, l_text);
    gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(l_dialog), &l_color);

    gtk_window_set_modal((GtkWindow*)l_dialog, 1);
    l_exit= gtk_dialog_run(GTK_DIALOG(l_dialog));

    if (GTK_RESPONSE_OK != l_exit)
    {
      break;
    }

    gtk_color_chooser_get_rgba(GTK_COLOR_CHOOSER(l_dialog), &l_color);

    l_rgba= gdk_rgba_to_string(&l_color);
    gtk_entry_set_text((*l_sum).m_fill_color_entry, l_rgba);
    g_free(l_rgba);

  }while(0);

  if (l_dialog)
  {
    gtk_widget_destroy((GtkWidget*)l_dialog);
  }
  
  return;
}

static void
item_fsum_font_button_cb(
  GtkWidget *                           i_widget,
  gpointer                              i_data)
{
  GtkWidget*                            l_dialog;
  int                                   l_exit;
  struct sum_dialog*                    l_sum;
  gchar*                                l_font;
  gchar const*                          l_text;

  l_dialog= 0;
  l_exit= 0;
  l_font= 0;

  do
  {

    l_sum= (struct sum_dialog*)i_data;

    l_text= gtk_entry_get_text(GTK_ENTRY((*l_sum).m_font_entry));

    l_dialog= gtk_font_chooser_dialog_new("Font", get_main_window());

    gtk_font_chooser_set_font(GTK_FONT_CHOOSER(l_dialog), l_text);

    gtk_window_set_modal((GtkWindow*)l_dialog, 1);
    l_exit= gtk_dialog_run(GTK_DIALOG(l_dialog));

    if (GTK_RESPONSE_OK != l_exit)
    {
      break;
    }

    l_font= gtk_font_chooser_get_font(GTK_FONT_CHOOSER(l_dialog));
    gtk_entry_set_text(
      (GTK_ENTRY((*l_sum).m_font_entry)),
      l_font);

  }while(0);

  if (l_font)
  {
    g_free(l_font);
  }

  if (l_dialog)
  {
    gtk_widget_destroy((GtkWidget*)l_dialog);
  }

  return;
}

static void
item_fsum_bind(
  struct sum_dialog*const               o_sum,
  GtkBuilder*const                      i_builder)
{

  memset(o_sum, 0, sizeof(*o_sum));

  do
  {

    (*o_sum).m_color_entry=
      GTK_ENTRY(gtk_builder_get_object(i_builder, "color_entry"));

    (*o_sum).m_color_button=
      GTK_BUTTON(gtk_builder_get_object(i_builder, "color_button"));

    g_signal_connect(
      (*o_sum).m_color_button,
      "clicked",
      G_CALLBACK(item_fsum_color_button_cb),
      o_sum);

    (*o_sum).m_font_entry=
      GTK_ENTRY(gtk_builder_get_object(i_builder, "font_entry"));

    (*o_sum).m_font_button=
      GTK_BUTTON(gtk_builder_get_object(i_builder, "font_button"));

    g_signal_connect(
      (*o_sum).m_font_button,
      "clicked",
      G_CALLBACK(item_fsum_font_button_cb),
      o_sum);

    (*o_sum).m_border_checkbutton=
      GTK_CHECK_BUTTON(gtk_builder_get_object(i_builder, "border_checkbutton"));

    (*o_sum).m_border_style_comboboxtext=
      GTK_COMBO_BOX_TEXT(gtk_builder_get_object(i_builder, "border_style_comboboxtext"));

    (*o_sum).m_border_thickness_spinbutton=
      GTK_SPIN_BUTTON(gtk_builder_get_object(i_builder, "border_thickness_spinbutton"));

    (*o_sum).m_border_color_entry=
      GTK_ENTRY(gtk_builder_get_object(i_builder, "border_color_entry"));

    (*o_sum).m_border_color_button=
      GTK_BUTTON(gtk_builder_get_object(i_builder, "border_color_button"));

    g_signal_connect(
      (*o_sum).m_border_color_button,
      "clicked",
      G_CALLBACK(item_fsum_border_color_button_cb),
      o_sum);

    (*o_sum).m_fill_color_checkbutton=
      GTK_CHECK_BUTTON(gtk_builder_get_object(i_builder, "fill_color_checkbutton"));

    (*o_sum).m_fill_color_button=
      GTK_BUTTON(gtk_builder_get_object(i_builder, "fill_color_button"));

    g_signal_connect(
      (*o_sum).m_fill_color_button,
      "clicked",
      G_CALLBACK(item_fsum_fill_color_button_cb),
      o_sum);

    (*o_sum).m_fill_color_entry=
      GTK_ENTRY(gtk_builder_get_object(i_builder, "fill_color_entry"));

    (*o_sum).m_type_comboboxtext=
      GTK_COMBO_BOX_TEXT(gtk_builder_get_object(i_builder, "type_comboboxtext"));

    (*o_sum).m_pattern_entry=
      GTK_ENTRY(gtk_builder_get_object(i_builder, "pattern_entry"));

    (*o_sum).m_locale_entry=                       
      GTK_ENTRY(gtk_builder_get_object(i_builder, "locale_entry"));

    (*o_sum).m_group_entry=
      GTK_ENTRY(gtk_builder_get_object(i_builder, "group_entry"));

    (*o_sum).m_field_entry=
      GTK_ENTRY(gtk_builder_get_object(i_builder, "field_entry"));

    (*o_sum).m_field_button=
      GTK_BUTTON(gtk_builder_get_object(i_builder, "field_button"));

    g_signal_connect(
      (*o_sum).m_field_button,
      "clicked",
      G_CALLBACK(item_fsum_field_button_cb),
      o_sum);

  }while(0);

  return;
}

static void
item_fsum_set_values(
  struct sum_dialog*const               i_sumd,
  struct item_fsum const*const           i_sum)
{
  gchar*                                l_color;
  gchar*                                l_font;

  l_color= gdk_rgba_to_string(&(*i_sum).m_color);
  gtk_entry_set_text((*i_sumd).m_color_entry, l_color);
  g_free(l_color);
  l_color= 0;

  l_font= pango_font_description_to_string((*i_sum).m_font);
  gtk_entry_set_text((*i_sumd).m_font_entry, l_font);
  g_free(l_font);

  gtk_toggle_button_set_active(
    GTK_TOGGLE_BUTTON((*i_sumd).m_border_checkbutton),
    (*i_sum).m_border.m_active);

  gtk_combo_box_set_active(
      GTK_COMBO_BOX((*i_sumd).m_border_style_comboboxtext), 
      (*i_sum).m_border.m_style);

  gtk_spin_button_set_value(
    GTK_SPIN_BUTTON((*i_sumd).m_border_thickness_spinbutton),
    (*i_sum).m_border.m_thickness);

  l_color= gdk_rgba_to_string(&(*i_sum).m_border.m_color);
  gtk_entry_set_text((*i_sumd).m_border_color_entry, l_color);
  g_free(l_color);
  l_color= 0;

  gtk_toggle_button_set_active(
    GTK_TOGGLE_BUTTON((*i_sumd).m_fill_color_checkbutton),
    (*i_sum).m_fill.m_active);

  l_color= gdk_rgba_to_string(&(*i_sum).m_fill.m_color);
  gtk_entry_set_text((*i_sumd).m_fill_color_entry, l_color);
  g_free(l_color);
  l_color= 0;

  gtk_combo_box_set_active(
      GTK_COMBO_BOX((*i_sumd).m_type_comboboxtext),
      (*i_sum).m_type);

  if ((*i_sum).m_pattern)
  {
    gtk_entry_set_text((*i_sumd).m_pattern_entry, (*i_sum).m_pattern);
  }

  if ((*i_sum).m_locale)
  {
    gtk_entry_set_text((*i_sumd).m_locale_entry, (*i_sum).m_locale);
  }

  gtk_entry_set_text((*i_sumd).m_group_entry, (*i_sum).m_group);
  gtk_entry_set_text((*i_sumd).m_field_entry, (*i_sum).m_tag);

  return;

}

static void
item_fsum_copy_values(
  struct item_fsum *const                io_sum,
  struct sum_dialog const*const         i_sumd)
{
  gchar const*                          l_text;
  gint                                  l_rc;

  l_text= gtk_entry_get_text(GTK_ENTRY((*i_sumd).m_color_entry));
  gdk_rgba_parse(&(*io_sum).m_color, l_text);

  l_text= gtk_entry_get_text(GTK_ENTRY((*i_sumd).m_font_entry));
  if ((*io_sum).m_font)
  {
    pango_font_description_free((*io_sum).m_font);
  }
  (*io_sum).m_font= pango_font_description_from_string(l_text);

  (*io_sum).m_border.m_active=
    gtk_toggle_button_get_active(
      GTK_TOGGLE_BUTTON((*i_sumd).m_border_checkbutton));

  l_rc = gtk_combo_box_get_active(
    GTK_COMBO_BOX((*i_sumd).m_border_style_comboboxtext));

  if ((0 <= l_rc) && (LINE_STYLE_SLOTS > l_rc))
  {
    (*io_sum).m_border.m_style= (enum line_style_type)l_rc;
  }
  else
  {
    (*io_sum).m_border.m_style= line_style_solid;
  }

  (*io_sum).m_border.m_thickness= gtk_spin_button_get_value(
    GTK_SPIN_BUTTON((*i_sumd).m_border_thickness_spinbutton));

  l_text= gtk_entry_get_text(GTK_ENTRY((*i_sumd).m_border_color_entry));
  gdk_rgba_parse(&(*io_sum).m_border.m_color, l_text);

  (*io_sum).m_fill.m_active=
    gtk_toggle_button_get_active(
      GTK_TOGGLE_BUTTON((*i_sumd).m_fill_color_checkbutton));

  l_text= gtk_entry_get_text(GTK_ENTRY((*i_sumd).m_fill_color_entry));
  gdk_rgba_parse(&(*io_sum).m_fill.m_color, l_text);

  l_rc = gtk_combo_box_get_active(
    GTK_COMBO_BOX((*i_sumd).m_type_comboboxtext));

  switch((enum number_type)l_rc)
  {
    case number_integer:
      (*io_sum).m_type= number_integer;
      break;
    case number_monetary:
      (*io_sum).m_type= number_monetary;
      break;
    default:
      (*io_sum).m_type= number_float;
      break;
  }

  if ((*io_sum).m_pattern)
  {
    g_free((*io_sum).m_pattern);
  }

  l_text= gtk_entry_get_text(GTK_ENTRY((*i_sumd).m_pattern_entry));
  (*io_sum).m_pattern= g_strdup(l_text);

  if ((*io_sum).m_locale)
  {
    g_free((*io_sum).m_locale);
  }

  l_text= gtk_entry_get_text(GTK_ENTRY((*i_sumd).m_locale_entry));
  (*io_sum).m_locale= g_strdup(l_text);

  l_text= gtk_entry_get_text(GTK_ENTRY((*i_sumd).m_group_entry));
  memset((*io_sum).m_group, 0, sizeof((*io_sum).m_group));
  strncpy((*io_sum).m_group, l_text, sizeof((*io_sum).m_group)-1);

  l_text= gtk_entry_get_text(GTK_ENTRY((*i_sumd).m_field_entry));
  memset((*io_sum).m_tag, 0, sizeof((*io_sum).m_tag));
  strncpy((*io_sum).m_tag, l_text, sizeof((*io_sum).m_tag)-1);
  g_strstrip((*io_sum).m_tag);

  return;
}

extern int
item_fsum_property(
  struct item_fsum*const                 io_sum)
{
  GtkBox*                               l_box;
  GtkBuilder*                           l_builder;
  GtkDialog*                            l_dialog;
  GtkGrid*                              l_grid;
  GError*                               l_error;
  int                                   l_exit;
  struct sum_dialog                     l_sum;
  int                                   l_rc;

  l_builder= 0;
  l_grid= 0;
  l_dialog= 0;
  l_error= 0;
  l_exit= 0;

  do
  {

    l_builder= gtk_builder_new();

    l_rc= wrap_gtk_builder_add_from_file(l_builder, "fsum.glade", &l_error);

    if (0 == l_rc)
    {
      l_exit= -1;
      break;
    }
  
    l_grid= GTK_GRID(gtk_builder_get_object(l_builder, "fsum_grid"));

    if (0 == l_grid)
    {
      l_error= g_error_new(
        GENERAL,
        GENERIC,
        "Unable to find dialog object: 'sum_grid'");
      l_exit= -1;
      break;
    }

    item_fsum_bind(&l_sum, l_builder);

    item_fsum_set_values(&l_sum, io_sum);

    l_dialog= GTK_DIALOG(gtk_dialog_new_with_buttons(
      "Sum Edit",
      get_main_window(),
      GTK_DIALOG_MODAL,
      "Ok",
      GTK_RESPONSE_OK,
      "Cancel",
      GTK_RESPONSE_CANCEL,
      NULL));

    l_box= GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(l_dialog)));

    gtk_box_pack_start(GTK_BOX(l_box), GTK_WIDGET(l_grid), 1, 1,0);

    gtk_widget_show(GTK_WIDGET(l_grid));

    gtk_widget_show_all(GTK_WIDGET(l_dialog));
    gtk_window_set_modal(GTK_WINDOW(l_dialog), 1);
    l_exit= gtk_dialog_run(l_dialog);

    if (GTK_RESPONSE_OK != l_exit)
    {
      break;
    }

    item_fsum_copy_values(io_sum, &l_sum);

  }while(0);

  if (l_builder)
  {
    g_object_unref(l_builder);
  }

  if (l_error)
  {
    _error_log(l_error);
    _error_display_prompt(l_error);
  }

  g_clear_error(&l_error);

  if (l_grid)
  {
    gtk_widget_destroy((GtkWidget*)l_dialog);
  }

  return l_exit;
}

